@@ -77,13 +77,11 @@ L<Filter::Crypto>, L<PAR::Filter::Crypto>
 
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
-You can write
+L<http://par.perl.org/> is the official PAR website.  You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
-Archives of the mailing list are available at
-E<lt>https://www.mail-archive.com/par@perl.org/E<gt> or E<lt>https://groups.google.com/g/perl.parE<gt>.
 
-Please submit bug reports to E<lt>https://github.com/rschupp/PAR-Packer/issuesE<gt>.
+Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
@@ -92,6 +90,6 @@ Copyright 2003-2009 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
-See F<LICENSE>.
+See L<http://www.perl.com/perl/misc/Artistic.html>
 
 =cut
