@@ -29,13 +29,13 @@ sub apply {
     $data = $1 if $$ref =~ s/((?:^__DATA__\r?\n).*)//ms;
 
     my $line = 1;
-    if ($$ref =~ /^=(?:head\d|pod|begin|item|over|for|back|end|cut|encoding)\b/) {
+    if ($$ref =~ /^=(?:head\d|pod|begin|item|over|for|back|end|cut)\b/) {
         $$ref = "\n$$ref";
         $line--;
     }
     $$ref =~ s{(
 	(.*?\n)
-	(?:=(?:head\d|pod|begin|item|over|for|back|end|encoding)\b
+	(?:=(?:head\d|pod|begin|item|over|for|back|end)\b
     .*?\n)
 	(?:=cut[\t ]*[\r\n]*?|\Z)
 	(\r?\n)?
@@ -46,6 +46,8 @@ sub apply {
 	) . $post;
     }gsex;
 
+    $$ref =~ s{^=encoding\s+\S+\s*$}{\n}mg;
+
     $$ref = '#line 1 "' . ($filename) . "\"\n" . $$ref
         if length $filename;
     $$ref =~ s/^#line 1 (.*\n)(#!.*\n)/$2#line 2 $1/g;
@@ -72,7 +74,7 @@ Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright 2003-2008 Audrey Tang E<lt>cpan@audreyt.orgE<gt>,
+Copyright 2003-2009 Audrey Tang E<lt>cpan@audreyt.orgE<gt>,
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
