@@ -1,12 +1,11 @@
 package PAR::StrippedPARL::Base;
-use 5.008001;
+use 5.006;
 use strict;
 use warnings;
 our $VERSION = '0.975';
 
 use File::Temp ();
 use File::Spec;
-use Cwd;
 use Config ();
 
 =head1 NAME
@@ -49,34 +48,44 @@ sub write_parl {
     my $file = shift;
     if (not defined $file) {
         warn "${class}->write_parl() needs a file name as argument";
-        return;
+        return();
     }
 
     # write out to a temporary file first
     my ($fh, $tfile) = File::Temp::tempfile(
-        "parlXXXX", SUFFIX => $Config::Config{_exe}||'', TMPDIR => 1, UNLINK => 1);
+        "parlXXXX",
+        SUFFIX => $Config::Config{_exe}||'',
+        DIR => File::Spec->tmpdir(),
+    );
     close $fh;
+    # File::Temp, you suck!
 
     if (not $class->write_raw($tfile)) {
         unlink($tfile);
         warn "Could not write temporary parl (class $class) to file '$tfile'";
-        return;
+        return();
     }
-    chmod(0755, $tfile);
+    chmod(oct('755'), $tfile);
 
     # Use this to generate a real parl
     my @libs = ();
     for my $ilib ( @INC ) {
-        next if ref $ilib;
         $ilib =~ s/\\$/\\\\/;
         push(@libs, qq(-I$ilib) );
     }
+    my @args = (@libs, qw/-q -B/);
+
+    # prepend ./ if applicable
+    my $execute = $tfile;
+    my @dirs = File::Spec->splitdir(
+        (File::Spec->splitpath($execute))[1] # directory part only
+    );
+    $execute = File::Spec->catfile(File::Spec->curdir(), $tfile) if not @dirs;
+    system($execute, @args, "-O$file") and unlink($tfile), return();
+    
+    # clean up
+    unlink($tfile);
 
-    system(Cwd::abs_path($tfile), @libs, qw( -q -B ), "-O$file");
-    unless ($? == 0) {
-        warn "Failed to execute temporary parl (class $class) in file '$tfile': $!";
-        return;
-    }
     return 1;
 }
 
@@ -158,7 +167,7 @@ Copyright 2006-2009 by Steffen Mueller E<lt>smueller@cpan.orgE<gt>.
 This program is free software; you can redistribute it and/or 
 modify it under the same terms as Perl itself.
 
-See F<LICENSE>.
+See L<http://www.perl.com/perl/misc/Artistic.html>
 
 =cut
 
