@@ -145,7 +145,9 @@ This is just a zip file beginning with the magic string "C<PK\003\004>".
 
 The pre-computed cache name.  A pack('Z40') string of the value of -T 
 (--tempcache) or the hash of the file, followed by C<\0CACHE>.  The hash
-of the file is calculated with L<Digest::SHA>.
+of the file is calculated with L<Digest::SHA>, L<Digest::SHA1>, or 
+L<Digest::MD5>.  If none of those modules is available, the C<mtime> of
+the file is used.
 
 A pack('N') number of the total length of FILE and PAR sections,
 followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
@@ -154,8 +156,7 @@ followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
 
 =cut
 
-
-my ($PAR_MAGIC, $par_temp, $progname, @tmpfile);
+my ($par_temp, $progname, @tmpfile);
 END { if ($ENV{PAR_CLEAN}) {
     require File::Temp;
     require File::Basename;
@@ -204,12 +205,21 @@ rm '" . $tmp->filename . "'
 
 BEGIN {
     Internals::PAR::BOOT() if defined &Internals::PAR::BOOT;
-    $PAR_MAGIC = "\nPAR.pm\n";
 
     eval {
 
 _par_init_env();
 
+if (exists $ENV{PAR_ARGV_0} and $ENV{PAR_ARGV_0} ) {
+    @ARGV = map $ENV{"PAR_ARGV_$_"}, (1 .. $ENV{PAR_ARGC} - 1);
+    $0 = $ENV{PAR_ARGV_0};
+}
+else {
+    for (keys %ENV) {
+        delete $ENV{$_} if /^PAR_ARGV_/;
+    }
+}
+
 my $quiet = !$ENV{PAR_DEBUG};
 
 # fix $progname if invoked from PATH
@@ -231,44 +241,21 @@ my ($start_pos, $data_pos);
     open _FH, '<', $progname or last;
     binmode(_FH);
 
-    # Search for the "\nPAR.pm\n signature backward from the end of the file
     my $buf;
-    my $size = -s $progname;
-    my $chunk_size = 64 * 1024;
-    my $magic_pos;
+    seek _FH, -8, 2;
+    read _FH, $buf, 8;
+    last unless $buf eq "\nPAR.pm\n";
 
-    if ($size <= $chunk_size) {
-        $magic_pos = 0;
-    } elsif ((my $m = $size % $chunk_size) > 0) {
-        $magic_pos = $size - $m;
-    } else {
-        $magic_pos = $size - $chunk_size;
-    }
-    # in any case, $magic_pos is a multiple of $chunk_size
-
-    while ($magic_pos >= 0) {
-        seek(_FH, $magic_pos, 0);
-        read(_FH, $buf, $chunk_size + length($PAR_MAGIC));
-        if ((my $i = rindex($buf, $PAR_MAGIC)) >= 0) {
-            $magic_pos += $i;
-            last;
-        }
-        $magic_pos -= $chunk_size;
-    }
-    last if $magic_pos < 0;
-
-    # Seek 4 bytes backward from the signature to get the offset of the 
-    # first embedded FILE, then seek to it
-    seek _FH, $magic_pos - 4, 0;
+    seek _FH, -12, 2;
+    read _FH, $buf, 4;
+    seek _FH, -12 - unpack("N", $buf), 2;
     read _FH, $buf, 4;
-    seek _FH, $magic_pos - 4 - unpack("N", $buf), 0;
-    $data_pos = tell _FH;
 
+    $data_pos = (tell _FH) - 4;
     # }}}
 
     # Extracting each file into memory {{{
     my %require_list;
-    read _FH, $buf, 4;                           # read the first "FILE"
     while ($buf eq "FILE") {
         read _FH, $buf, 4;
         read _FH, $buf, unpack("N", $buf);
@@ -282,13 +269,24 @@ my ($start_pos, $data_pos);
         read _FH, $buf, unpack("N", $buf);
 
         if (defined($ext) and $ext !~ /\.(?:pm|pl|ix|al)$/i) {
-            my $filename = _tempfile("$crc$ext", $buf, 0755);
+            my ($out, $filename) = _tempfile($ext, $crc);
+            if ($out) {
+                binmode($out);
+                print $out $buf;
+                close $out;
+                chmod 0755, $filename;
+            }
             $PAR::Heavy::FullCache{$fullname} = $filename;
             $PAR::Heavy::FullCache{$filename} = $fullname;
         }
         elsif ( $fullname =~ m|^/?shlib/| and defined $ENV{PAR_TEMP} ) {
-            my $filename = _tempfile("$basename$ext", $buf, 0755);
+            # should be moved to _tempfile()
+            my $filename = "$ENV{PAR_TEMP}/$basename$ext";
             outs("SHLIB: $filename\n");
+            open my $out, '>', $filename or die $!;
+            binmode($out);
+            print $out $buf;
+            close $out;
         }
         else {
             $require_list{$fullname} =
@@ -307,21 +305,32 @@ my ($start_pos, $data_pos);
 
         return if ref $module or !$module;
 
-        my $info = delete $require_list{$module} or return;
+        my $filename = delete $require_list{$module} || do {
+            my $key;
+            foreach (keys %require_list) {
+                next unless /\Q$module\E$/;
+                $key = $_; last;
+            }
+            delete $require_list{$key} if defined($key);
+        } or return;
 
-        $INC{$module} = "/loader/$info/$module";
+        $INC{$module} = "/loader/$filename/$module";
 
         if ($ENV{PAR_CLEAN} and defined(&IO::File::new)) {
             my $fh = IO::File->new_tmpfile or die $!;
             binmode($fh);
-            print $fh $info->{buf};
+            print $fh $filename->{buf};
             seek($fh, 0, 0);
             return $fh;
         }
         else {
-            my $filename = _tempfile("$info->{crc}.pm", $info->{buf});
-
-            open my $fh, '<', $filename or die "can't read $filename: $!";
+            my ($out, $name) = _tempfile('.pm', $filename->{crc});
+            if ($out) {
+                binmode($out);
+                print $out $filename->{buf};
+                close $out;
+            }
+            open my $fh, '<', $name or die $!;
             binmode($fh);
             return $fh;
         }
@@ -360,7 +369,7 @@ my ($start_pos, $data_pos);
     # }}}
 
     last unless $buf eq "PK\003\004";
-    $start_pos = (tell _FH) - 4;                # start of zip
+    $start_pos = (tell _FH) - 4;
 }
 # }}}
 
@@ -448,7 +457,6 @@ if ($out) {
         #local $INC{'Cwd.pm'} = __FILE__ if $^O ne 'MSWin32';
         require IO::File;
         require Archive::Zip;
-        require Digest::SHA;
     }
 
     my $par = shift(@ARGV);
@@ -497,7 +505,7 @@ if ($out) {
     my $loader = scalar <_FH>;
     if (!$ENV{PAR_VERBATIM} and $loader =~ /^(?:#!|\@rem)/) {
         require PAR::Filter::PodStrip;
-        PAR::Filter::PodStrip->apply(\$loader, $0);
+        PAR::Filter::PodStrip->new->apply(\$loader, $0)
     }
     foreach my $key (sort keys %env) {
         my $val = $env{$key} or next;
@@ -516,23 +524,30 @@ if ($out) {
     if ($bundle) {
         require PAR::Heavy;
         PAR::Heavy::_init_dynaloader();
-
         init_inc();
 
         require_modules();
 
-        my @inc = grep { !/BSDPAN/ } 
-                       grep {
-                           ($bundle ne 'site') or
-                           ($_ ne $Config::Config{archlibexp} and
-                           $_ ne $Config::Config{privlibexp});
-                       } @INC;
+        my @inc = sort {
+            length($b) <=> length($a)
+        } grep {
+            !/BSDPAN/
+        } grep {
+            ($bundle ne 'site') or
+            ($_ ne $Config::Config{archlibexp} and
+             $_ ne $Config::Config{privlibexp});
+        } @INC;
 
-        # Now determine the files loaded above by require_modules():
-        # Perl source files are found in values %INC and DLLs are
-        # found in @DynaLoader::dl_shared_objects.
+        if ($Config{_delim} eq '\\') { s{\\}{/}g for @inc }
+
+        # File exists test added to fix RT #41790:
+        # Funny, non-existing entry in _<....auto/Compress/Raw/Zlib/autosplit.ix.
+        # This is a band-aid fix with no deeper grasp of the issue.
+        # Somebody please go through the pain of understanding what's happening,
+        # I failed. -- Steffen
         my %files;
-        $files{$_}++ for @DynaLoader::dl_shared_objects, values %INC;
+        /^_<(.+)$/ and -e $1 and $files{$1}++ for keys %::;
+        $files{$_}++ for values %INC;
 
         my $lib_ext = $Config::Config{lib_ext};
         my %written;
@@ -578,10 +593,12 @@ if ($out) {
                 $content = <FILE>;
                 close FILE;
 
-                PAR::Filter::PodStrip->apply(\$content, "<embedded>/$name")
+                PAR::Filter::PodStrip->new->apply(\$content, $file)
                     if !$ENV{PAR_VERBATIM} and $name =~ /\.(?:pm|ix|al)$/i;
 
-                PAR::Filter::PatchContent->new->apply(\$content, $file, $name);
+                # Do not let XSLoader pick up auto/* from environment
+                $content =~ s/goto +retry +unless +.*/goto retry;/
+                    if lc($name) eq lc("XSLoader.pm");
             }
 
             outs(qq(Written as "$name"));
@@ -601,19 +618,30 @@ if ($out) {
 
     $cache_name = substr $cache_name, 0, 40;
     if (!$cache_name and my $mtime = (stat($out))[9]) {
-        my $ctx = Digest::SHA->new(1);
-        open(my $fh, "<", $out);
-        binmode($fh);
-        $ctx->addfile($fh);
-        close($fh);
+        my $ctx = eval { require Digest::SHA; Digest::SHA->new(1) }
+            || eval { require Digest::SHA1; Digest::SHA1->new }
+            || eval { require Digest::MD5; Digest::MD5->new };
 
-        $cache_name = $ctx->hexdigest;
+        # Workaround for bug in Digest::SHA 5.38 and 5.39
+        my $sha_version = eval { $Digest::SHA::VERSION } || 0;
+        if ($sha_version eq '5.38' or $sha_version eq '5.39') {
+            $ctx->addfile($out, "b") if ($ctx);
+        }
+        else {
+            if ($ctx and open(my $fh, "<$out")) {
+                binmode($fh);
+                $ctx->addfile($fh);
+                close($fh);
+            }
+        }
+
+        $cache_name = $ctx ? $ctx->hexdigest : $mtime;
     }
     $cache_name .= "\0" x (41 - length $cache_name);
     $cache_name .= "CACHE";
     $fh->print($cache_name);
     $fh->print(pack('N', $fh->tell - length($loader)));
-    $fh->print($PAR_MAGIC);
+    $fh->print("\nPAR.pm\n");
     $fh->close;
     chmod 0755, $out;
     # }}}
@@ -638,16 +666,10 @@ if ($out) {
         require File::Find;
         require Archive::Zip;
     }
-
-    my $fh = IO::File->new;                             # Archive::Zip operates on an IO::Handle
-    $fh->fdopen(fileno(_FH), 'r') or die "$!: $@";
-
-    # Temporarily increase the chunk size for Archive::Zip so that it will find the EOCD
-    # even if lots of stuff has been appended to the pp'ed exe (e.g. by OSX codesign).
-    Archive::Zip::setChunkSize(-s _FH);
     my $zip = Archive::Zip->new;
+    my $fh = IO::File->new;
+    $fh->fdopen(fileno(_FH), 'r') or die "$!: $@";
     $zip->readFromFileHandle($fh, $progname) == Archive::Zip::AZ_OK() or die "$!: $@";
-    Archive::Zip::setChunkSize(64 * 1024);
 
     push @PAR::LibCache, $zip;
     $PAR::LibCache{$progname} = $zip;
@@ -726,20 +748,16 @@ sub require_modules {
     require IO::File;
     require Compress::Zlib;
     require Archive::Zip;
-    require Digest::SHA;
     require PAR;
     require PAR::Heavy;
     require PAR::Dist;
     require PAR::Filter::PodStrip;
-    require PAR::Filter::PatchContent;
-    require attributes;
     eval { require Cwd };
     eval { require Win32 };
     eval { require Scalar::Util };
     eval { require Archive::Unzip::Burst };
     eval { require Tie::Hash::NamedCapture };
     eval { require PerlIO; require PerlIO::scalar };
-    eval { require utf8 };
 }
 
 # The C version of this code appears in myldr/mktmpdir.c
@@ -771,7 +789,7 @@ sub _set_par_temp {
         }
         $username =~ s/\W/_/g;
 
-        my $stmpdir = "$path$Config{_delim}par-".unpack("H*", $username);
+        my $stmpdir = "$path$Config{_delim}par-$username";
         mkdir $stmpdir, 0755;
         if (!$ENV{PAR_CLEAN} and my $mtime = (stat($progname))[9]) {
             open (my $fh, "<". $progname);
@@ -784,18 +802,24 @@ sub _set_par_temp {
                 $stmpdir .= "$Config{_delim}cache-" . $buf;
             }
             else {
-                my $digest = eval 
-                {
-                    require Digest::SHA; 
-                    my $ctx = Digest::SHA->new(1);
-                    open(my $fh, "<", $progname);
-                    binmode($fh);
-                    $ctx->addfile($fh);
-                    close($fh);
-                    $ctx->hexdigest;
-                } || $mtime;
+                my $ctx = eval { require Digest::SHA; Digest::SHA->new(1) }
+                    || eval { require Digest::SHA1; Digest::SHA1->new }
+                    || eval { require Digest::MD5; Digest::MD5->new };
 
-                $stmpdir .= "$Config{_delim}cache-$digest"; 
+                # Workaround for bug in Digest::SHA 5.38 and 5.39
+                my $sha_version = eval { $Digest::SHA::VERSION } || 0;
+                if ($sha_version eq '5.38' or $sha_version eq '5.39') {
+                    $ctx->addfile($progname, "b") if ($ctx);
+                }
+                else {
+                    if ($ctx and open(my $fh, "<$progname")) {
+                        binmode($fh);
+                        $ctx->addfile($fh);
+                        close($fh);
+                    }
+                }
+
+                $stmpdir .= "$Config{_delim}cache-" . ( $ctx ? $ctx->hexdigest : $mtime );
             }
             close($fh);
         }
@@ -812,33 +836,23 @@ sub _set_par_temp {
     $par_temp = $1 if $ENV{PAR_TEMP} and $ENV{PAR_TEMP} =~ /(.+)/;
 }
 
-
-# check if $name (relative to $par_temp) already exists;
-# if not, create a file with a unique temporary name, 
-# fill it with $contents, set its file mode to $mode if present;
-# finaly rename it to $name; 
-# in any case return the absolute filename
 sub _tempfile {
-    my ($name, $contents, $mode) = @_;
+    my ($ext, $crc) = @_;
+    my ($fh, $filename);
 
-    my $fullname = "$par_temp/$name";
-    unless (-e $fullname) {
-        my $tempname = "$fullname.$$";
+    $filename = "$par_temp/$crc$ext";
 
-        open my $fh, '>', $tempname or die "can't write $tempname: $!";
-        binmode $fh;
-        print $fh $contents;
-        close $fh;
-        chmod $mode, $tempname if defined $mode;
-
-        rename($tempname, $fullname) or unlink($tempname);
-        # NOTE: The rename() error presumably is something like ETXTBSY 
-        # (scenario: another process was faster at extraction $fullname
-        # than us and is already using it in some way); anyway, 
-        # let's assume $fullname is "good" and clean up our copy.
+    if ($ENV{PAR_CLEAN}) {
+        unlink $filename if -e $filename;
+        push @tmpfile, $filename;
+    }
+    else {
+        return (undef, $filename) if (-r $filename);
     }
 
-    return $fullname;
+    open $fh, '>', $filename or die $!;
+    binmode($fh);
+    return($fh, $filename);
 }
 
 # same code lives in PAR::SetupProgname::set_progname
@@ -895,7 +909,7 @@ sub _par_init_env {
         $ENV{PAR_INITIALIZED} = 2;
     }
 
-    for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ) ) {
+    for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ARGC ARGV_0 ) ) {
         delete $ENV{'PAR_'.$_};
     }
     for (qw/ TMPDIR TEMP CLEAN DEBUG /) {
@@ -966,11 +980,11 @@ L<PAR>, L<PAR::Dist>, L<parl>, L<pp>
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>,
 Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
-You can write
+L<http://par.perl.org/> is the official PAR website.  You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
 
-Please submit bug reports to E<lt>bug-par-packer@rt.cpan.orgE<gt>.
+Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
@@ -993,7 +1007,7 @@ under the Artistic License.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
-See F<LICENSE>.
+See L<http://www.perl.com/perl/misc/Artistic.html>
 
 =cut
 
